unit UFile;
//*****************************************************************************
// Unit UFile           - Diese Unit enthlt alle fr das Fighter-Spiel
// -------------          ntige Funktionen zu Dateibearbeitung bereit
//                      - Lesen und Schreiben der Benutzereinstellungen bzw. 
//                        Highscore in die Config-Datei
//                      - Lesen und Schreiben der Spiele-Objekte (Spieler,
//                        Gegner, Schuss...)
//
// ffentliche-Funktionen:
// -----------------------
// ClearAttributes      - Entfernt vorhandene Dateiattribute bis auf "Archiv"
// DelAttributes        - Entfernt vorhandene Dateiattribute bis auf "Archiv"
// LoadHighscore        - Ldt die Highscore aus der Config-Datei
// IsNewHighscore       - Prft, ob der Spieler gut genug fr die Highscore war
// SortHighscore        - Sortiert eine Highscore-Liste nach den Punkten
// AddToHighscore       - Fgt einen neuen Wert (Name+Punkte) in die Highscore
// SaveHighscore        - Speichert die Highscore in einer Config-Datei
//
// SaveGameInfo         - Speichert die Benutzereinstellungen aus einer Game-
//                        Struktur in eine Config-Datei
// LoadGameInfo         - Ldt die in einer Config-Datei gespeicherten
//                        Benutzereinstellungen in eine Game-Struktur
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UExtImgLst           - Enthlt Routinen fr eine Bilderliste
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses Classes, Dialogs, Graphics, Controls, IniFiles, SysUtils, UType, UExtImgLst;

function  LoadGameInfo(var Game : TGame; FileName : String) : Boolean;
procedure SaveGameInfo(var Game : TGame; FileName : String);

function  LoadHighscore(var Highscore : THighscore; FileName : String) : Boolean;
function  IsNewHighscore(FileName : String; PlayerPoints : Cardinal) : Boolean;
procedure SortHighscore(var Score : THighscore);
procedure AddToHighscore(FileName : String; PlayerName : TPlayerName; PlayerPoints : Cardinal);
procedure SaveHighscore(Highscore : THighscore; FileName : String);

function  SaveGameBitmaps(GameBmp : TExtImgLst; FileName : TFileName) : Boolean;
function  LoadGameBitmaps(FileName : TFileName) : TExtImgLst;

implementation




//*****************************************************************************
// Entfernt alle vorhandene Attribute einer Datei und setzt sie "Archiv"
//
//  ->   FileName    Name der zu bearbeitenden Datei
//  ->   MustExists  Fehler wenn Datei noch nicht existiert?
// <                 Gibt zurck, ob das Entfernen geklappt hat oder nicht
//                   True = okay, False = Fehler
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function ClearAttributes(FileName : String; MustExisits : Boolean) : Boolean;
var
 Attrs : Integer;
begin
 ClearAttributes := True;
 if FileExists(FileName) then                                     // existiert Datei berhaupt?
 begin
    Attrs := FileGetAttr(FileName);                               // ermittel Dateiattribute

    if Attrs and faReadOnly <> 0 then                             // Wenn Datei schreibgeschtzt
       if MessageDlg('Soll der Schreibschutz der Datei: '+FileName+' entfernt werden?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
       begin
          if FileSetAttr(FileName, faArchive) <> 0 then           // Versuche Dateischutz zu entfernen
          begin
             ShowMessage(C_ERR_SET_FILE_ATTR+FileName);
             ClearAttributes := False;
          end;
       end
       else
          ClearAttributes := False;
 end
 else
 begin
    if MustExisits then                                           // wenn Fehler ausgegeben werden soll
       ShowMessage('Fehler: Datei '+FileName+' existiert nicht!');

    ClearAttributes := not MustExisits;                           // beende mit enspr. Flag
 end;
end;


//*****************************************************************************
// Ldt eine Config Datei, liest den Inhalt aus und speichert ihn
// in einer Game-Struktur ab
//
// <->   Game        Die Game-Struktur, die den Inhalt aufnimmt
//  ->   FileName    Name der Config-Datei
// <                 Gibt zurck, ob das Laden geklappt hat oder nicht
//                   True = okay, False = Fehler
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
function LoadGameInfo(var Game : TGame; FileName : String) : Boolean;
var
  IniFile : TIniFile;
begin
 LoadGameInfo := False;
 try
       // ffne Ini-Datei
       IniFile := TIniFile.Create(FileName);
       with Game do
       begin
          FieldSize   := C_DEF_PLAYFIELD;
          Status      := St_Playing;

          // Allgemein
          LevelTime   := IniFile.ReadInteger('General', 'LevelTime', C_DEF_LEVELTIME);
          Lives       := IniFile.ReadInteger('General', 'Lives',     C_DEF_LIVES);
          NiceLook    := IniFile.ReadBool   ('General', 'NiceLook',  FALSE);

          // Sterne
          StCount     := IniFile.ReadInteger('Stars',   'Count',     60);
          StMaxSpeed  := IniFile.ReadInteger('Stars',   'MaxSpeed',  12);

          // Tasten
          CtrLeft     := IniFile.ReadInteger('Control', 'Left',      C_DEF_LEFT);
          CtrRight    := IniFile.ReadInteger('Control', 'Right',     C_DEF_RIGHT);
          CtrFire     := IniFile.ReadInteger('Control', 'Fire',      C_DEF_FIRE);
          CtrPause    := IniFile.ReadInteger('Control', 'Pause',     C_DEF_PAUSE);

          // Gegner
          EnPoints    := IniFile.ReadInteger('Enemy',   'Points',    100);
          EnCount     := IniFile.ReadInteger('Enemy',   'Count',     3);
          EnIncCount  := IniFile.ReadInteger('Enemy',   'IncCount',  2);
          EnSpeed     := IniFile.ReadInteger('Enemy',   'Speed',     5);
          EnIncSpeed  := IniFile.ReadInteger('Enemy',   'IncSpeed',  1);
          EnFire      := IniFile.ReadInteger('Enemy',   'Fire',      8);
          EnIncFire   := IniFile.ReadInteger('Enemy',   'IncFire',   2);
          if not FileExists(FileName) then         // Wenn Datei gar nicht existiert,
             ShowMessage(FileName+C_ERR_LOAD);     // gebe Meldung aus und verwende Standardwerte

          LoadGameInfo := True;
       end;

 except
      on EIniFileException do
         ShowMessage(C_ERR_LOAD_OPTIONS+FileName);
 end;
 IniFile.Free;

end;


//*****************************************************************************
// Speichert den Inhalt einer Game-Struktur in einer Config-Datei ab
//
// <->   Game        Game-Struktur, die abgespeichert werden soll
//  ->   FileName    Name der Config-Datei
//
// as, 25.09.2002 (22:18 Uhr) - Erste Version
//*****************************************************************************
procedure SaveGameInfo(var Game : TGame; FileName : String);
var
  IniFile : TIniFile;
begin
 // Entferne alle Datei-Attribute
 if ClearAttributes(FileName, False) then
 begin
    try
       // ffne Ini-Datei
       IniFile := TIniFile.Create(FileName);
       with Game do
       begin
          // Allgemein
          IniFile.WriteInteger('General', 'LevelTime', LevelTime);
          IniFile.WriteInteger('General', 'Lives',     Lives);
          IniFile.WriteBool   ('General', 'NiceLook',  NiceLook);


          // Sterne
          IniFile.WriteInteger('Stars',   'Count',     StCount);
          IniFile.WriteInteger('Stars',   'MaxSpeed',  StMaxSpeed);

          // Tasten
          IniFile.WriteInteger('Control', 'Left',      CtrLeft);
          IniFile.WriteInteger('Control', 'Right',     CtrRight);
          IniFile.WriteInteger('Control', 'Fire',      CtrFire);
          IniFile.WriteInteger('Control', 'Pause',     CtrPause);

          // Gegner
          IniFile.WriteInteger('Enemy',   'Points',    EnPoints);
          IniFile.WriteInteger('Enemy',   'Count',     EnCount);
          IniFile.WriteInteger('Enemy',   'IncCount',  EnIncCount);
          IniFile.WriteInteger('Enemy',   'Speed',     EnSpeed);
          IniFile.WriteInteger('Enemy',   'IncSpeed',  EnIncSpeed);
          IniFile.WriteInteger('Enemy',   'Fire',      EnFire);
          IniFile.WriteInteger('Enemy',   'IncFire',   EnIncFire);
       end;
    except
      on EIniFileException do
         ShowMessage(C_ERR_SAVE_OPTIONS+FileName);
    end;
    IniFile.Free;
 end
 else
    ShowMessage(C_ERR_SAVE_OPTIONS+FileName);

end;


//*****************************************************************************
// Ldt die Highscore (jeweils Spielernamen und Punkte) aus einer Config-
// Datei in eine Highscore-Struktur
//
//  ->   FileName    Name der Config-Datei
//  ->   Highscore   Highscore-Record, dass die Liste aus der Datei aufnimmt
// <                 Gibt zurck, ob das Laden geklappt hat oder nicht
//                   True = okay, False = Fehler
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function LoadHighscore(var Highscore : THighscore; FileName : String) : Boolean;
var
  IniFile       : TIniFile;
  i             : 1..C_MAX_HIGHSCORE_ENTRIES;
begin
 LoadHighscore := False;
 try
    // ffne Ini-Datei
    IniFile := TIniFile.Create(FileName);

    // Durchlaufe jeden Eintrag der Highscore
    for i:=1 to C_MAX_HIGHSCORE_ENTRIES do
    begin
       // Lese den Spielernamen und die erreichten Punkte aus der Datei
       Highscore[i].PlayerName   := IniFile.ReadString('Highscore', 'PlayerName'+IntToStr(i),    C_DEF_PLAYERNAME);
       Highscore[i].PlayerPoints := IniFile.ReadInteger('Highscore', 'PlayerPoints'+IntToStr(i), 1000    );
    end;
    if not FileExists(FileName) then         // Wenn Datei gar nicht existiert,
       ShowMessage(FileName+C_ERR_LOAD);     // gebe Meldung aus und verwende Standardwerte
    LoadHighscore := True;

 except
   on EIniFileException do
      ShowMessage(C_ERR_LOAD_HIGHSCORE+FileName);

 end;

 IniFile.Free;
end;



//*****************************************************************************
// Prft, ob der Spieler gut genug fr die Highscore war
//
//  ->   FileName       Name der Datei, in der die Highscore gespeichert ist
//  ->   PlayerPoints   Erreichte Spielerpunktzahl
// <                    Gibt zurck, ob die Punktzahl fr die Highscore reicht
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function IsNewHighscore(FileName : String; PlayerPoints : Cardinal) : Boolean;
var
 Score : THighscore;
begin
 if LoadHighscore(Score,FileName) then       // Lade Highscore-Liste
    IsNewHighscore := PlayerPoints > Score[C_MAX_HIGHSCORE_ENTRIES].PlayerPoints
 else
    IsNewHighscore := False;                 // Falls Fehler beim Laden -> kein neuer Eintrag
end;


//*****************************************************************************
// Sortiert eine Highscore-Liste nach den Punkten mit dem
// Bubble-Sort-Verfahren
//
// <->   Score       Zu sortierendes Record mit der Highscore-Liste
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure SortHighscore(var Score : THighscore);
var
 i,j  : Cardinal;
 k    : TScoreElement;
begin
 for j:=C_MAX_HIGHSCORE_ENTRIES-1 downto 1 do   // Durchlaufe Liste nach unten
    for i:=1 to j do                            // und vergleiche jeweils zwei Elemente
       if Score[i].PlayerPoints < Score[i+1].PlayerPoints then
          begin
          k          := Score[i+1];             // Tausche zwei Elemente
          Score[i+1] := Score[i];
          Score[i]   := k;
          end;
end;


//*****************************************************************************
// Fgt einen neuen Wert (Name+Punkte) in die Highscore, wenn die Punktzahl
// gro genug ist
//
//  ->   FileName     Name der Datei aus der die Highscore stammt
//  ->   PlayerName   Neuer Name des Spielers
//  <-   PlayerPoints Erreichte Punktzahl des Spielers
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure AddToHighscore(FileName : String; PlayerName : TPlayerName; PlayerPoints : Cardinal);
var
 Score : THighscore;
begin
 if LoadHighscore(Score, FileName) then
    if PlayerPoints > Score[C_MAX_HIGHSCORE_ENTRIES].PlayerPoints then
    begin
       Score[C_MAX_HIGHSCORE_ENTRIES].PlayerName   := PlayerName;    // Der letzte fliegt raus
       Score[C_MAX_HIGHSCORE_ENTRIES].PlayerPoints := PlayerPoints;  // erstmal ist neuer Eintrag der letzte
       SortHighscore(Score);                                         // dann wird die Highscore neu sortiert
       SaveHighscore(Score,FileName);                                // und abgespeichert
    end;
end;


//*****************************************************************************
// Speichert eine Highscore in einer Config-Datei
//
//  ->   Highscore   Highscore-Werte, die gespeichert werden sollen
//  ->   FileName    Name der Config-Datei
//
// as, 26.09.2002 (23:39 Uhr) - Erste Version
//*****************************************************************************
procedure SaveHighscore(Highscore : THighscore; FileName : String);
var
  IniFile       : TIniFile;
  i             : 1..C_MAX_HIGHSCORE_ENTRIES;
begin
 // Entfernen alle Datei-Attribute
 if ClearAttributes(FileName, False) then
 begin
    try
       // ffne Ini-Datei
       IniFile := TIniFile.Create(FileName);

       // Durchlaufe jeden Eintrag der Highscore
       for i:=1 to C_MAX_HIGHSCORE_ENTRIES do
       begin
          // Speicher den Spielernamen und die erreichten Punkte
          IniFile.WriteString ('Highscore', 'PlayerName'+IntToStr(i),   Highscore[i].PlayerName);
          IniFile.WriteInteger('Highscore', 'PlayerPoints'+IntToStr(i), Highscore[i].PlayerPoints);
       end;

    except
      on EIniFileException do
         ShowMessage(C_ERR_SAVE_HIGHSCORE+FileName);

    end;
    IniFile.Free;
  end
  else
     ShowMessage(C_ERR_SAVE_HIGHSCORE+FileName);
end;


//*****************************************************************************
// Speichert Bitmap und zugehrige Gre (in % auf Spielfeld) als Block
// am Ende einer bereits geffneten Datei
//
// <->   f           Handle auf bereits zum Schreiben geffnete Datei
//  ->   Size        Gre (in %) die das Bild auf dem Spielfeld belegen soll
// <->   Bmp         Bitmap, das abgespeichert werden soll
//
// Achtung:          Fehler werden durch Exception abgefangen und an
//                   die aufrufende Routine durchgereicht!
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure WriteBmpBlock(var f : File; Size : Byte; var Bmp : TBitmap);
var
 StreamSize    : Int64;
 ms            : TMemoryStream;
begin
 try
    ms := TMemoryStream.Create;                 // erzeuge Platz im Speicher und
    Bmp.SaveToStream(ms);                       // speicher Bitmap im RAM als Stream
    StreamSize:=ms.Size;                        // ermittle Stream-Gre in Bytes
    BlockWrite(f, Size,   Sizeof(Byte) );       // speicher Prozent-Information im ersten Byte
    BlockWrite(f, StreamSize, Sizeof(Int64) );  // speicher Stream-Gre dahinter
    Blockwrite(f, ms.Memory^, StreamSize);      // speicher Bitmap Stream in Datei
    ms.Free;
 except
    ms.Free;                                    // Speicher wieder freigeben
    Raise;                                      // Exception durchreichen
 end;
end;


//*****************************************************************************
// Ldt ein Bitmap und die zugehrige Gre (in % auf Spielfeld) als Block
// aus einer bereits geffneten Datei (von aktuellen Lese-Position)
//
// <->   f           Handle auf bereits zum Lesen geffnete Datei
//  ->   Size        Gre (in %) die das Bild auf dem Spielfeld belegen soll
// <                 Gibt Bitmap zurck, das aus Datei gelesen wurde
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function ReadBmpBlock(var f : File; var Size : Byte) : TBitmap;
var
 Bmp           : TBitmap;
 StreamSize    : Int64;
 ms            : TMemoryStream;
begin
 try
    ms  := TMemoryStream.Create;                   // erzeuge Platz im Speicher und
    Bmp := TBitmap.Create;                         // und erzeuge neues Bitmap-Objekt
    BlockRead(f, Size, Sizeof(Byte) );             // lese Gre des Bildes (prozentual)
    BlockRead(f, StreamSize, Sizeof(Int64) );      // wie gro ist der folgende Bmp-Stream?
    ms.SetSize(StreamSize);                        // Passe Stream-Gre an
    Blockread(f, ms.Memory^, StreamSize);          // lese Bmp-Stream aus Datei in den Speicher
    Bmp.LoadFromStream(ms);                        // Wandel Memory-Stream in Bild um
    ReadBmpBlock := Bmp;                           // Gebe Bitmap zurck
    ms.Free;                                       // Speicher wieder freigeben
 except
    on Exception do
    begin
       ms.Free;                                    // Speicher wieder freigeben
       Bmp.Free;
       ReadBmpBlock := nil;                        // Fehler = nil zurckgeben
       Raise;                                      // Exception durchreichen
    end;
 end;

end;


//*****************************************************************************
// Speichert die Spiele-Grafiken in einer Datei ab
//
//  ->   GameBmp     Bilder-Liste, die alle Spiele-Grafiken enthlt
//  ->   FileName    Name der Datei, in die gespeichert wird
//
// <                 Gibt zurck, ob das Speichern geklappt hat!
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function SaveGameBitmaps(GameBmp : TExtImgLst; FileName : TFileName) : Boolean;
var
 f    : File;
 i    : Word;
 Size : Byte;
 Bmp  : TBitmap;
begin
 // Entfernen alle Datei-Attribute
 SaveGameBitmaps := False;
 if ClearAttributes(FileName, False) then
 begin

    try
       AssignFile(f, FileName);
       Rewrite(f,1);                         // Datei ffnen
       for i:=1 to GameBmp.Count do          // durchlaufe jedes Bild der Spiele-Grafiken
       begin
          Bmp := GameBmp.GetBmp(i, Size);    // merke jeweils ein Bild
          WriteBmpBlock(f, Size, Bmp);       // und speicher es als Block in die Datei
       end;
       CloseFile(f);                         // Gebe Datei wieder frei!
       SaveGameBitmaps := True;              // Kein Fehler -> Rckgabe = True

    except
       on Exception do
          SaveGameBitmaps := False;
                                             // Bmp muss nicht freigegeben werden, da Zeiger
    end;

 end
 else
    SaveGameBitmaps := False;                // Gebe Fehler zurck

end;


//*****************************************************************************
// Ldt die Spiele-Grafiken aus einer Datei in eine Bilder-Liste
//
//  ->   FileName    Name der Datei aus der gelesen wird
// <                 Gibt die Bilderliste, die aus der Datei gelesen wurde
//
// Achtung:          Wenn ein Fehler auftritt, wird dieser abgefangen, aber
//                   nicht ausgegeben, die Bilderliste wird nicht erzeugt,
//                   es wird nil zurckgegeben!
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function LoadGameBitmaps(FileName : TFileName) : TExtImgLst;
var
 f       : File;
 ImgLst  : TExtImgLst;
 Size    : Byte;
 Bmp     : TBitmap;
begin
 // Entfernen alle Datei-Attribute
 if ClearAttributes(FileName, True) then
 begin
 try
       ImgLst := TExtImgLst.Create;                // erzeuge Bilderlisten-Objekt zur Aufnahme
       Bmp    := TBitmap.Create;
       ImgLst.Init;

       AssignFile(f, FileName);
       Reset(f,1);                                 // ffne Datei, aus der gelesen wird

       while not Eof(f) do                         // Lese bis zum Dateiende
       begin
          Bmp.Assign( ReadBmpBlock(f, Size) );     // jeweils ein Bild nach dem anderen
          ImgLst.AddBmp(Bmp, Size);                // und speichere es in der Bilderliste
       end;

       if ImgLst.Count < 4 then                    // Falls zu wenig Bilder in der Datei sind
       begin
          ImgLst.AllFree;                          // Entferne alle Bilder -> Fehler
          ImgLst.Free;
       end;
       LoadGameBitmaps := ImgLst;                   // Rckgabe ist die fertige Bilderliste
       Bmp.Free;                                    // gebe Bild-Buffer frei

    except
       on Exception do                             // Falls Lesefehler auftritt
       begin
          CloseFile(f);                            // Schliee Datei
          Bmp.Free;                                // und gebe Bild-Buffer frei
          ImgLst.AllFree;                          // entferne diese aus der Bilderliste
          ImgLst.Free;                             // und gebe diese frei
          LoadGameBitmaps := nil;                  // gebe nil=Fehler zurck
       end;

    end;
 end
 else
   LoadGameBitmaps := nil;                      // gebe nil=Fehler zurck

end;


end.
