unit UHighscore;
//*****************************************************************************
// Unit UHighscore      - Diese Formular-Unit enthlt alle Routinen zur
// -------------          Anzeige der Highscore-Liste
//
// ffentliche-Methoden:
// ---------------------
// SetHighscore         - Setzt die Highscore-Liste, die angezeigt
//                        wird sobald die Form sichtbar wird
//
// Verwendet eigene Units:
// -----------------------
// UType                - enthlt wichtige Typen und Konstanten wie z.B.
//                        THighscore, TPlayerName etc.
// UFile                - enthlt Routinen zum Lesen der Highscore-Liste aus
//                        der Datei
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, StdCtrls, UType;

type
  THighscoreForm = class(TForm)
    BtnOk: TButton;
    LaCaption: TLabel;
    LaPlace: TLabel;
    LaName: TLabel;
    LaPoints: TLabel;
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    g_Highscore : THighscore;
    function  LongPlayerName(PlayerName : TPlayerName) : TPlayerName;
    procedure InitHighscore(Highscore : THighscore);
  public
    { Public-Deklarationen }
    procedure SetHighscore(Highscore : THighscore);
  end;

var
  HighscoreForm: THighscoreForm;

implementation

uses UFile;

{$R *.dfm}

//*****************************************************************************
// Fgt dem Spieler-Namen zur besseren Anzeige Punkte hinten an
//
//  ->   PlayerName  String mit dem Namen des Spielers
// <                 Gibt den Namen mit angefgten Punkten zurck
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
function THighscoreForm.LongPlayerName(PlayerName : TPlayerName) : TPlayerName;
begin
 while Length(PlayerName) < C_MAX_PLAYERNAME_LEN do               // Durchlaufe bis zur max. Spieler-Namen-Lnge
    PlayerName := PlayerName + '.';                               // und fge Punkt hinten an den Namen
 LongPlayerName := PlayerName;                                    // Gebe genderten Namen zurck
end;


//*****************************************************************************
// Zeigt eine Highscore-Liste mit den entspr. Controls an
//
//  ->   Highscore   Highscore-Record mit den Namen / Punkten der Spieler
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure THighscoreForm.InitHighscore(Highscore : THighscore);
var
 i : 1..C_MAX_HIGHSCORE_ENTRIES;
 StrPlace, StrName, StrPoints : String;
begin
 for i:=1 to C_MAX_HIGHSCORE_ENTRIES do                           // durchlaufe jeden Eintrag (Namen/Punkte) der Liste
 begin
   // jeder Eintrag ist eine Zeile (Abschluss mit #13)
   StrPlace  := StrPlace  + IntToStr(i) + '.'                       + #13;   // Welcher Platz in der Highscore?
   StrName   := StrName   + LongPlayerName(Highscore[i].PlayerName) + #13;   // Name des Spielers
   StrPoints := StrPoints + IntToStr(Highscore[i].PlayerPoints)     + #13;   // Erreichte Punktzahl des Spielers
 end;
 LaPlace.Caption  := StrPlace;                                    // Setze die Controls mit richtigen Werten
 LaName.Caption   := StrName;
 LaPoints.Caption := StrPoints;
end;


//*****************************************************************************
// Setzt die Highscore-Liste, die angezeigt wird sobald die Form sichtbar wird
//
//  ->   Highscore   Highscore-Liste, die angezeigt werden soll
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure THighscoreForm.SetHighscore(Highscore : THighscore);
begin
 g_Highscore := Highscore;
end;


//*****************************************************************************
// Initialisiert und zeigt alle Controls mit der Highscore-Liste
// Wird aufgerufen, sobald die Highscore-Form angezeigt werden soll
// Zuvor muss mit SetHighscore die Highscore-Liste initialisiert werden
//
// as, 26.09.2002 (23:16 Uhr) - Erste Version
//*****************************************************************************
procedure THighscoreForm.FormShow(Sender: TObject);
begin
 InitHighscore(g_Highscore);          // zeigt aktuelles Highscore-Record an
end;

end.
