unit UObjStar;
//*****************************************************************************
// Unit UObjStar        - Diese Unit enthlt die Klasse TStars inklusive
// -------------          aller Deklarationen und Member-Methoden
//                      - Die Klasse TStars dient zur Anzeige und Bewegung
//                        der Sterne im Hintergrund des Spielfelds.
//                      - Sterne werden einfach als Rechtecke gezeichnet
//                        (das spart etwas rechenaufwand und ist kaum sichtbar)
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert die Sternen-Objekt-Liste mit gewhlter
//                        Anzahl an Sternen
// Draw                 - Zeichnet alle Sterne aus der Liste und kmmert
//                        sich autom. um deren Bewegung nach unten
// Resize               - Korrigiert die Positionen und Gre der Sterne
//                        nach dem ndern der Spielfeldgre
// FreeLst              - Gibt die Liste mit allen Sternen-Obj. wieder frei
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UObjFighter          - Enthlt die Elternklasse "TFighterObject", von der
//                        sich diese Klasse ableitet
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface
uses Types, Graphics, UType, UObjFighter;

type

TStars = class(TFighterObject)
                private
                ListPtr   : TFighterPtr;           // Zeiger auf den Anfang der Liste mit den Sternen
                MaxSpeed  : Byte;                  // Maximale Geschwindigkeit der Sterne
                MaxWidth  : Byte;                  // Maximale Breite eines Sternes
                MaxHeight : Byte;                  // Maximale Hhe eines Sternes
                function  RandomStar(Max_x, Max_y : integer) : TFighterRec;

                public
                procedure Init(Count : Word; MaxSp : Byte; Max_x, Max_y : integer);
                procedure Draw(Canvas : TCanvas);
                procedure Resize(OldField, NewField : Integer);
                procedure FreeLst;
         end;

implementation

//*****************************************************************************
// Gibt einem Stern zufllige Werte fr Posititon, Farbe, Hhe/Breite und
// Geschwindigkeit
//
//  ->   Max_x       Maximale Spielfeldbreite
//  ->   Max_y       Maximale Spielfeldhhe
// <                 Gibt ein Stern-Record mit den zuflligen Werten zurck
//
// as, 20.09.2002 - Erste Version
//*****************************************************************************
function TStars.RandomStar(Max_x, Max_y : integer) : TFighterRec;
var
 Star   : TFighterRec;
begin
 with Star do                                                     // benutze das Sternen-Record
 begin
    x        := Random(Max_x)+1;                                  // Zuflliger X-Pos im Spielfeld
    y        := Random(Max_y)+1;                                  // Zuflliger Y-Pos im Spielfeld
    Color    := C_STARCOLORS[ Random(High(C_STARCOLORS)) +1 ];    // Suche Farbe fr Stern aus vordef. aus
    Speed    := Random(MaxSpeed)+1;                               // zufllige Geschwindigkeit
    Height   := Random(MaxHeight)+1;                              // zufllige Hhe des Sternes
    Width    := Random(MaxWidth)+1;                               // zufllige Breite des Sternes
 end;
 RandomStar  := Star;
end;


//*****************************************************************************
// Initialisiert die Sternen-Liste mit beliebiger Anzahl an Sternen
//
//  ->   Count       Anzahl der Sterne, die erscheinen sollen
//  ->   MaxSp       Maximale Geschwindigkeit eines Sterns
//  ->   Max_x       Spielfeldbreite
//  ->   Max_y       Spielfeldhhe
//
// as, 20.09.2002 - Erste Version
//*****************************************************************************
procedure TStars.Init(Count : Word; MaxSp : Byte; Max_x, Max_y : Integer);
begin
 ListPtr   := nil;                              // Liste ist noch leer
 MaxSpeed  := MaxSp;                            // setze maximale Speed
 MaxWidth  := Max_x div 300;                    // und max. Breite fr Stern
 MaxHeight := Max_y div 300;                    // und max. Hhe fr Stern
 while (Count > 0) do                           // fge Count viele Sterne hinzu
   begin
   Add(ListPtr, RandomStar(Max_x, Max_y)) ;     // Stern mit zuflligen Werten wird angehngt
   dec(Count);
   end;
end;


//*****************************************************************************
// Bewegt und zeichnet alle Sterne der Sternen-Liste. Sterne, die unten aus
// dem Spielfeld verschwinden, werden oben wieder eingefgt
//
//  ->   Canvas      Zeichenflche auf der die Sterne gezeichnet werden
//
// as, 20.09.2002 - Erste Version
//*****************************************************************************
procedure TStars.Draw(Canvas : TCanvas);
var
   tmpPtr   : TFighterPtr;
   Rect     : TRect;
begin
 tmpPtr := ListPtr;                                               // gehe zum Listenanfang
 while tmpPtr <> nil do                                           // durchlaufe jedes Stern-Objekt
 begin

   with tmpPtr^.Data do                                           // greife auf Stern-Infos zu
      begin
      if (y > Canvas.ClipRect.Bottom) or (y < 0) then             // wenn Stern nicht mehr im Spielfeld
         tmpPtr^.Data := RandomStar(Canvas.ClipRect.Right, 1)     // nder seine Werte und fge ihn wieder oben ein
      else
         inc(y,Speed);                                            // bewege Stern nach unten, je nach Geschw.

      Rect.Left   := x;                                           // baue Sternen-Rechteck zusammen aus X-Position
      Rect.Top    := y;                                           // Y-Position
      Rect.Right  := x+Width;                                     // Stern-Breite
      Rect.Bottom := y+Height;                                    // und Stern-Hhe
      Canvas.Brush.Color := Color;                                // richtige Farbe fr Stern setzen
      Canvas.FillRect(Rect);                                      // und Stern als Rechteck zeichnen (Ellipse ist
                                                                  // zu rechenaufwendig und kaum sehbar)
      end;
   tmpPtr := tmpPtr^.Next;                                        // gehe zum nchsten Stern
 end;
end;


//*****************************************************************************
// Passt jeden Stern an eine neue Spielfeldgre an, indem Position und
// Gre neu berechnet werden
//
//  ->   OldField    alte Spielfeldgre
//  ->   NewField    neue Spielfeldgre
//
// as, 20.09.2002 - Erste Version
//*****************************************************************************
procedure TStars.Resize(OldField, NewField : Integer);
var
   tmpPtr   : TFighterPtr;
begin
 tmpPtr := ListPtr;
 while tmpPtr <> nil do                               // durchlaufe jeden Stern
   begin
   MaxWidth  := NewField div 300;                     // berechne neue Sternen-Breite
   MaxHeight := NewField div 300;                     // berechne neue Sternen-Hhe
   with tmpPtr^.Data do
      begin
      x     := PosChange(OldField, NewField, x);      // berechne neue X-Position des Sterns
      y     := PosChange(OldField, NewField, y);      // berechne neue Y-Position des Sterns
      Height:= Random(MaxHeight)+1;                   // berechne neue Stern-Hhe
      Width := Random(MaxWidth)+1;                    // berechne neue Stern-Breite
      end;
   tmpPtr := tmpPtr^.Next;                            // gehe zum nchsten Stern
   end;
end;


//*****************************************************************************
// Entfernt alle Sterne der Liste udn gibt den Speicher wieder frei
//
// as, 20.09.2002 - Erste Version
//*****************************************************************************
procedure TStars.FreeLst;
begin
 AllFree(ListPtr);                                    // geerbte Routine erledigt die Freigabe
end;


end.
