unit UOptions;
//*****************************************************************************
// Unit UOptions        - Diese Unit ist die Formular-Unit des Optionen-Dialogs
// -------------          Sie enthlt alle Routinen zum Anzeigen, Bearbeiten,
//                        Laden und Speichern der Optionen
//
// ffentliche-Methoden:
// ---------------------
// SetGameInfo          - Setzt die Voreinstellungen fr die Optionen
// SetGamePath          - Setzt den zu verwendenen Pfad des Spiels
//
// Verwendet eigene Units:
// -----------------------
// UType                - enthlt wichtige Typen und Konstanten wie z.B.
//                        THighscore, TPlayerName etc.
// UFile                - enthlt Routinen zum Lesen der Highscore-Liste aus
//                        der Datei
// UGame                - Enthlt allgemeine Routinen fr das Spiel
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Spin, IniFiles, UType;

type
  TOptionsForm = class(TForm)
    GrStars: TGroupBox;
    GrGeneral: TGroupBox;
    EdStCount: TSpinEdit;
    EdStMaxSpeed: TSpinEdit;
    LaAnzahl: TLabel;
    LaSpeed: TLabel;
    LaRoundTime: TLabel;
    LaPlayerLives: TLabel;
    GrEnemy: TGroupBox;
    EdHours: TSpinEdit;
    EdMinutes: TSpinEdit;
    EdSeconds: TSpinEdit;
    EdLives: TSpinEdit;
    LaStd: TLabel;
    LaMin: TLabel;
    LaSek: TLabel;
    LaEnLevel: TLabel;
    LaIncEnLevel: TLabel;
    LaSpeedLevel: TLabel;
    LaIncSpeedLevel: TLabel;
    LaShoot: TLabel;
    LaIncShoot: TLabel;
    GrControl: TGroupBox;
    LaLinks: TLabel;
    LaRechts: TLabel;
    LaSchuss: TLabel;
    LaPause: TLabel;
    EdEnCount: TSpinEdit;
    EdEnIncCount: TSpinEdit;
    EdEnSpeed: TSpinEdit;
    EdEnIncSpeed: TSpinEdit;
    EdEnFire: TSpinEdit;
    EdEnIncFire: TSpinEdit;
    BtnSave: TButton;
    BtnCancel: TButton;
    LaEnPoints: TLabel;
    EdEnPoints: TSpinEdit;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    EdLeft: TEdit;
    EdRight: TEdit;
    EdPause: TEdit;
    EdFire: TEdit;
    ChkNiceLook: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure ShowKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure BtnSaveClick(Sender: TObject);
  private
    { Private-Deklarationen }
    GameInfo   : TGame;
    g_GamePath : String;
    function  Key2String(Key : Word) : String;
    function  ChangesOk : Boolean;
    procedure InitOptions;
    procedure SaveOptions(FileName : String);
    procedure CheckToSave;
  public
    { Public-Deklarationen }
    procedure SetGameInfo(Game : TGame);
    procedure SetGamePath(Path : String);
  end;

var
  OptionsForm: TOptionsForm;

implementation


uses UGame, UFile;
{$R *.dfm}


//*****************************************************************************
// Ermittelt anhand eines Tastencodes, um welche Taste es sich handelt und
// gibt dies als Text wieder zurck
//
//  ->   Key         Tastencode
// <                 gibt die Taste als String zurck
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
function TOptionsForm.Key2String(Key : Word) : String;
begin
 case Key of
  // Welche Taste soll verwendet werden???
  VK_SPACE:       Key2String := 'Space';
  VK_BACK:        Key2String := 'Back';
  VK_TAB:         Key2String := 'Tab';
  VK_RETURN:      Key2String := 'Enter';
  VK_PAUSE:       Key2String := 'Pause';
  VK_CAPITAL:     Key2String := 'Caps';
  VK_ESCAPE:      Key2String := 'Esc';
  VK_PRIOR:	      Key2String := 'Bild auf';
  VK_NEXT:	      Key2String := 'Bild ab';
  VK_END:         Key2String := 'Ende';
  VK_HOME:        Key2String := 'Pos 1';
  VK_LEFT:	      Key2String := 'Links';
  VK_UP:          Key2String := 'Hoch';
  VK_RIGHT:       Key2String := 'Rechts';
  VK_DOWN:        Key2String := 'Runter';
  VK_INSERT:	   Key2String := 'Einf';
  VK_DELETE:      Key2String := 'Entf';
  VK_LWIN:        Key2String := 'Win links';
  VK_RWIN:        Key2String := 'Win rechts';
  VK_LSHIFT:      Key2String := 'Shift links';
  VK_RSHIFT:      Key2String := 'Shift rechts';
  VK_LCONTROL:    Key2String := 'Strg links';
  VK_RCONTROL:    Key2String := 'Strg rechts';
  VK_LMENU:       Key2String := 'Alt links';
  VK_RMENU:       Key2String := 'Alt rechts';
 else
 begin
    if Chr(Key) in ['a'..'z','A'..'Z','0'..'9'] then  // Die Standard-Tasten
       Key2String := Chr(Key)
    else
       Key2String := 'VK: '+IntToStr(Key);            // Unbekannte Taste -> Tastencode-Nr.
    end;
 end;
end;


//*****************************************************************************
// berprft die Eingaben des Benutzers, ob z.B. zwei Gleiche Tasten zur
// Steuerung des Spieler-Raumschiffs eingegeben wurden
//
// <                 Gibt zurck, ob die Eingaben okay sind ( = True)
//                   oder nicht ( = False)
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
function  TOptionsForm.ChangesOk : Boolean;
var
 Ctr   : array[1..4] of Word;
 i,f   : Byte;
 count : Byte;
begin
 Ctr[1] := EdLeft.Tag;                          // merke alle vier Steuer-Tasten
 Ctr[2] := EdRight.Tag;
 Ctr[3] := EdPause.Tag;
 Ctr[4] := EdFire.Tag;
 Count := 0;
 for i:=1 to 4 do                               // Vergleiche jede Taste mit
    for f:=1 to 4 do                            // allen anderen Tasten
       if Ctr[i] = Ctr[f] then Inc(Count);      // und merke bereinstimmung
 ChangesOk := Count = 4;                        // jede Taste nur einmal ( = 4) da?
end;


//*****************************************************************************
// Initialisiert alle Controls mit den Voreinstellungen aus der Config-Datei
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.InitOptions;
var
 PlayTime   : TPlayTime;
begin
 with GameInfo do
 begin
       // Allgemein
       PlayTime            := SekToPlayTime(LevelTime);
       EdHours.Value       := PlayTime.Hours;
       EdMinutes.Value     := PlayTime.Minutes;
       EdSeconds.Value     := PlayTime.Seconds;
       EdLives.Value       := Lives;
       ChkNiceLook.Checked := NiceLook;

       // Sterne
       EdStCount.Value     := StCount;
       EdStMaxSpeed.Value  := StMaxSpeed;

       // Tasten
       EdLeft.Text         := Key2String(CtrLeft);
       EdRight.Text        := Key2String(CtrRight);
       EdFire.Text         := Key2String(CtrFire);
       EdPause.Text        := Key2String(CtrPause);
       EdLeft.Tag          := CtrLeft;
       EdRight.Tag         := CtrRight;
       EdFire.Tag          := CtrFire;
       EdPause.Tag         := CtrPause;

       // Gegner
       EdEnPoints.Value    := EnPoints;
       EdEnCount.Value     := EnCount;
       EdEnIncCount.Value  := EnIncCount;
       EdEnSpeed.Value     := EnSpeed;
       EdEnIncSpeed.Value  := EnIncSpeed;
       EdEnFire.Value      := EnFire;
       EdEnIncFire.Value   := EnIncFire;
 end;
end;

//*****************************************************************************
// Ermittelt die neuen Einstellungen und speichert sie in der Config-Datei ab
//
//  ->   Game        zu verwendene Game-Struktur
//  ->   FileName    Name der Config-Datei
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.SaveOptions(FileName : String);
var
 PlayTime   : TPlayTime;
 Game       : TGame;
begin

 with Game do                    // flle das Game-Record mit den neuen Werten
 begin
    // Allgemein
    PlayTime.Hours      := EdHours.Value;
    PlayTime.Minutes    := EdMinutes.Value;
    PlayTime.Seconds    := EdSeconds.Value;
    LevelTime           := PlayTimeToSek(PlayTime);
    Lives               := EdLives.Value;
    NiceLook            := ChkNiceLook.Checked;

    // Sterne
    StCount             := EdStCount.Value;
    StMaxSpeed          := EdStMaxSpeed.Value;

    // Tasten
    CtrLeft             := EdLeft.Tag;
    CtrRight            := EdRight.Tag;
    CtrFire             := EdFire.Tag;
    CtrPause            := EdPause.Tag;

    // Gegner
    EnPoints            := EdEnPoints.Value;
    EnCount             := EdEnCount.Value;
    EnIncCount          := EdEnIncCount.Value;
    EnSpeed             := EdEnSpeed.Value;
    EnIncSpeed          := EdEnIncSpeed.Value;
    EnFire              := EdEnFire.Value;
    EnIncFire           := EdEnIncFire.Value;
 end;

 SaveGameInfo(Game, FileName);   // Speicher die neuen Werte in der Datei
end;


//*****************************************************************************
// Prft ob das Speichern mglichist und speichert die neuen
// Einstellungen nach Besttigung in der Config Datei
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.CheckToSave;
begin
 if ChangesOk then                                    // sind die Benutzer-nderungen gltig?
 begin
    if MessageDlg(C_MSG_SAVE_OPTIONS, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
       SaveOptions(g_GamePath+C_CONFIG_FILE);          // speicher Optionen in Datei (bei Fehler -> Meldung)
       ModalResult := mrOk;                           // beende das Optionen Dialogfeld
    end;
 end
 else
    ShowMessage(C_ERR_WRONG_OPTIONS);
end;


//*****************************************************************************
// Initialisiert alle Controls mit den Voreinstellungen aus der Config-Datei
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.FormShow(Sender: TObject);
begin
 InitOptions;
end;


//*****************************************************************************
// Zeigt im Editfeld an, welche Taste der Anwender gerae gedrckt hat
// Wird nach jedem Tastendruck in einem der vier Edits aufgerufen
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.ShowKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 with (Sender as TEdit) do       // aus welchem Edit-Feld kommt der Tastendruck
 begin
   Tag  := Key;                  // merke Tastencode
   Text := Key2String(Key);      // zeige an, welche Taste gedrckt wurde
 end;
end;


//*****************************************************************************
// Speichert die neuen Einstellungen nach Besttigung in der Config Datei
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.BtnSaveClick(Sender: TObject);
begin
 CheckToSave;
end;


//*****************************************************************************
// Setzt die Voreinstellungen fr die Optionen
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.SetGameInfo(Game : TGame);
begin
 GameInfo := Game;
end;


//*****************************************************************************
// Setzt den Pfad zur Exe-Datei Fighter.exe
//
//  ->   Path        Vollstndiger Pfad zur fighter.exe
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TOptionsForm.SetGamePath(Path : String);
begin
 g_GamePath := Path;
end;


end.
